@@ -1,14 +1,18 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, BangPatterns #-}
 {-# OPTIONS_HADDOCK prune #-}
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
 
 -- |
 -- Module      : Data.ByteString.Lazy.Char8
--- Copyright   : (c) Don Stewart 2006
+-- Copyright   : (c) Don Stewart 2006-2008
+--               (c) Duncan Coutts 2006-2011
 -- License     : BSD-style
 --
--- Maintainer  : dons@cse.unsw.edu.au
--- Stability   : experimental
--- Portability : non-portable (imports Data.ByteString.Lazy)
+-- Maintainer  : dons00@gmail.com, duncan@community.haskell.org
+-- Stability   : stable
+-- Portability : portable
 --
 -- Manipulate /lazy/ 'ByteString's using 'Char' operations. All Chars will
 -- be truncated to 8 bits. It can be expected that these functions will
@@ -20,6 +24,11 @@
 --
 -- > import qualified Data.ByteString.Lazy.Char8 as C
 --
+-- The Char8 interface to bytestrings provides an instance of IsString
+-- for the ByteString type, enabling you to use string literals, and
+-- have them implicitly packed to ByteStrings.
+-- Use @{-\# LANGUAGE OverloadedStrings \#-}@ to enable this.
+--
 
 module Data.ByteString.Lazy.Char8 (
 
@@ -33,6 +42,8 @@ module Data.ByteString.Lazy.Char8 (
         unpack,                 -- :: ByteString -> String
         fromChunks,             -- :: [Strict.ByteString] -> ByteString
         toChunks,               -- :: ByteString -> [Strict.ByteString]
+        fromStrict,             -- :: Strict.ByteString -> ByteString
+        toStrict,               -- :: ByteString -> Strict.ByteString
 
         -- * Basic interface
         cons,                   -- :: Char -> ByteString -> ByteString
@@ -43,6 +54,7 @@ module Data.ByteString.Lazy.Char8 (
         uncons,                 -- :: ByteString -> Maybe (Char, ByteString)
         last,                   -- :: ByteString -> Char
         tail,                   -- :: ByteString -> ByteString
+        unsnoc,                 -- :: ByteString -> Maybe (ByteString, Char)
         init,                   -- :: ByteString -> ByteString
         null,                   -- :: ByteString -> Bool
         length,                 -- :: ByteString -> Int64
@@ -104,6 +116,8 @@ module Data.ByteString.Lazy.Char8 (
         groupBy,                -- :: (Char -> Char -> Bool) -> ByteString -> [ByteString]
         inits,                  -- :: ByteString -> [ByteString]
         tails,                  -- :: ByteString -> [ByteString]
+        stripPrefix,            -- :: ByteString -> ByteString -> Maybe ByteString
+        stripSuffix,            -- :: ByteString -> ByteString -> Maybe ByteString
 
         -- ** Breaking into many substrings
         split,                  -- :: Char -> ByteString -> [ByteString]
@@ -117,7 +131,7 @@ module Data.ByteString.Lazy.Char8 (
 
         -- * Predicates
         isPrefixOf,             -- :: ByteString -> ByteString -> Bool
---      isSuffixOf,             -- :: ByteString -> ByteString -> Bool
+        isSuffixOf,             -- :: ByteString -> ByteString -> Bool
 
         -- * Searching ByteStrings
 
@@ -155,6 +169,9 @@ module Data.ByteString.Lazy.Char8 (
         readInteger,
 
         -- * I\/O with 'ByteString's
+        -- | ByteString I/O uses binary mode, without any character decoding
+        -- or newline conversion. The fact that it does not respect the Handle
+        -- newline mode is considered a flaw and may be changed in a future version.
 
         -- ** Standard input and output
         getContents,            -- :: IO ByteString
@@ -172,17 +189,23 @@ module Data.ByteString.Lazy.Char8 (
         hGet,                   -- :: Handle -> Int64 -> IO ByteString
         hGetNonBlocking,        -- :: Handle -> Int64 -> IO ByteString
         hPut,                   -- :: Handle -> ByteString -> IO ()
+        hPutNonBlocking,        -- :: Handle -> ByteString -> IO ByteString
+        hPutStr,                -- :: Handle -> ByteString -> IO ()
+        hPutStrLn,              -- :: Handle -> ByteString -> IO ()
 
   ) where
 
 -- Functions transparently exported
 import Data.ByteString.Lazy 
-        (fromChunks, toChunks
+        (fromChunks, toChunks, fromStrict, toStrict
         ,empty,null,length,tail,init,append,reverse,transpose,cycle
-        ,concat,take,drop,splitAt,intercalate,isPrefixOf,group,inits,tails,copy
+        ,concat,take,drop,splitAt,intercalate
+        ,isPrefixOf,isSuffixOf,group,inits,tails,copy
+        ,stripPrefix,stripSuffix
         ,hGetContents, hGet, hPut, getContents
-        ,hGetNonBlocking
-        ,putStr, putStrLn, interact)
+        ,hGetNonBlocking, hPutNonBlocking
+        ,putStr, hPutStr, interact
+        ,readFile,writeFile,appendFile)
 
 -- Functions we need to wrap.
 import qualified Data.ByteString.Lazy as L
@@ -203,23 +226,7 @@ import Prelude hiding
         ,readFile,writeFile,appendFile,replicate,getContents,getLine,putStr,putStrLn
         ,zip,zipWith,unzip,notElem,repeat,iterate,interact,cycle)
 
-import System.IO            (hClose,openFile,IOMode(..))
-#ifndef __NHC__
-import Control.Exception    (bracket)
-#else
-import IO                   (bracket)
-#endif
-
-#if __GLASGOW_HASKELL__ >= 608
-import Data.String
-#endif
-
-#define STRICT1(f) f a | a `seq` False = undefined
-#define STRICT2(f) f a b | a `seq` b `seq` False = undefined
-#define STRICT3(f) f a b c | a `seq` b `seq` c `seq` False = undefined
-#define STRICT4(f) f a b c d | a `seq` b `seq` c `seq` d `seq` False = undefined
-#define STRICT5(f) f a b c d e | a `seq` b `seq` c `seq` d `seq` e `seq` False = undefined
-#define STRICT5_(f) f a b c d _ | a `seq` b `seq` c `seq` d `seq` False = undefined
+import System.IO            (Handle, stdout)
 
 ------------------------------------------------------------------------
 
@@ -228,27 +235,23 @@ singleton :: Char -> ByteString
 singleton = L.singleton . c2w
 {-# INLINE singleton #-}
 
-#if __GLASGOW_HASKELL__ >= 608
-instance IsString ByteString where
-    fromString = pack
-    {-# INLINE fromString #-}
-#endif
-
 -- | /O(n)/ Convert a 'String' into a 'ByteString'. 
 pack :: [Char] -> ByteString
-pack = L.pack. List.map c2w
+pack = packChars
 
 -- | /O(n)/ Converts a 'ByteString' to a 'String'.
 unpack :: ByteString -> [Char]
-unpack = List.map w2c . L.unpack
-{-# INLINE unpack #-}
+unpack = unpackChars
+
+infixr 5 `cons`, `cons'` --same as list (:)
+infixl 5 `snoc`
 
 -- | /O(1)/ 'cons' is analogous to '(:)' for lists.
 cons :: Char -> ByteString -> ByteString
 cons = L.cons . c2w
 {-# INLINE cons #-}
 
--- | /O(1)/ Unlike 'cons', 'cons\'' is
+-- | /O(1)/ Unlike 'cons', 'cons'' is
 -- strict in the ByteString that we are consing onto. More precisely, it forces
 -- the head and the first chunk. It does this because, for space efficiency, it
 -- may coalesce the new byte onto the first \'chunk\' rather than starting a
@@ -256,7 +259,7 @@ cons = L.cons . c2w
 --
 -- So that means you can't use a lazy recursive contruction like this:
 --
--- > let xs = cons\' c xs in xs
+-- > let xs = cons' c xs in xs
 --
 -- You can however use 'cons', as well as 'repeat' and 'cycle', to build
 -- infinite lazy ByteStrings.
@@ -284,6 +287,14 @@ uncons bs = case L.uncons bs of
                   Just (w, bs') -> Just (w2c w, bs')
 {-# INLINE uncons #-}
 
+-- | /O(n\/c)/ Extract the 'init' and 'last' of a ByteString, returning Nothing
+-- if it is empty.
+unsnoc :: ByteString -> Maybe (ByteString, Char)
+unsnoc bs = case L.unsnoc bs of
+                  Nothing -> Nothing
+                  Just (bs', w) -> Just (bs', w2c w)
+{-# INLINE unsnoc #-}
+
 -- | /O(1)/ Extract the last element of a packed string, which must be non-empty.
 last :: ByteString -> Char
 last = w2c . L.last
@@ -308,7 +319,7 @@ foldl :: (a -> Char -> a) -> a -> ByteString -> a
 foldl f = L.foldl (\a c -> f a (w2c c))
 {-# INLINE foldl #-}
 
--- | 'foldl\'' is like foldl, but strict in the accumulator.
+-- | 'foldl'' is like foldl, but strict in the accumulator.
 foldl' :: (a -> Char -> a) -> a -> ByteString -> a
 foldl' f = L.foldl' (\a c -> f a (w2c c))
 {-# INLINE foldl' #-}
@@ -326,7 +337,7 @@ foldl1 :: (Char -> Char -> Char) -> ByteString -> Char
 foldl1 f ps = w2c (L.foldl1 (\x y -> c2w (f (w2c x) (w2c y))) ps)
 {-# INLINE foldl1 #-}
 
--- | 'foldl1\'' is like 'foldl1', but strict in the accumulator.
+-- | 'foldl1'' is like 'foldl1', but strict in the accumulator.
 foldl1' :: (Char -> Char -> Char) -> ByteString -> Char
 foldl1' f ps = w2c (L.foldl1' (\x y -> c2w (f (w2c x) (w2c y))) ps)
 
@@ -724,7 +735,7 @@ we need a similarly lazy, but efficient version.
 -- after appending a terminating newline to each.
 unlines :: [ByteString] -> ByteString
 unlines [] = empty
-unlines ss = (concat $ List.intersperse nl ss) `append` nl -- half as much space
+unlines ss = concat (List.intersperse nl ss) `append` nl -- half as much space
     where nl = singleton '\n'
 
 -- | 'words' breaks a ByteString up into a list of words, which
@@ -745,13 +756,8 @@ unwords = intercalate (singleton ' ')
 -- there is no integer at the beginning of the string, it returns
 -- Nothing, otherwise it just returns the int read, and the rest of the
 -- string.
-
-{-
--- Faster:
-
-data MaybeS = NothingS
-            | JustS {-# UNPACK #-} !Int {-# UNPACK #-} !ByteString
--}
+--
+-- Note: This function will overflow the Int for large integers.
 
 readInt :: ByteString -> Maybe (Int, ByteString)
 {-# INLINE readInt #-}
@@ -763,8 +769,7 @@ readInt (Chunk x xs) = case w2c (B.unsafeHead x) of
 
     where loop :: Bool -> Int -> Int
                 -> S.ByteString -> ByteString -> Maybe (Int, ByteString)
-          STRICT5_(loop)
-          loop neg i n c cs
+          loop neg !i !n !c cs
               | B.null c = case cs of
                              Empty          -> end  neg i n c  cs
                              (Chunk c' cs') -> loop neg i n c' cs'
@@ -778,10 +783,10 @@ readInt (Chunk x xs) = case w2c (B.unsafeHead x) of
 
           {-# INLINE end #-}
           end _   0 _ _  _ = Nothing
-          end neg _ n c cs = e `seq` e
+          end neg _ n c cs = e
                 where n' = if neg then negate n else n
                       c' = chunk c cs
-                      e  = n' `seq` c' `seq` Just $! (n',c')
+                      e  = n' `seq` c' `seq` Just (n',c')
          --                  in n' `seq` c' `seq` JustS n' c'
 
 
@@ -809,8 +814,7 @@ readInteger (Chunk c0 cs0) =
 
           loop :: Int -> Int -> [Integer]
                -> S.ByteString -> ByteString -> (Integer, ByteString)
-          STRICT5_(loop)
-          loop d acc ns c cs
+          loop !d !acc ns !c cs
               | B.null c = case cs of
                              Empty          -> combine d acc ns c cs
                              (Chunk c' cs') -> loop d acc ns c' cs'
@@ -838,21 +842,16 @@ readInteger (Chunk c0 cs0) =
           end n c cs = let c' = chunk c cs
                         in c' `seq` (n, c')
 
--- | Read an entire file /lazily/ into a 'ByteString'. Use 'text mode'
--- on Windows to interpret newlines
-readFile :: FilePath -> IO ByteString
-readFile f = openFile f ReadMode >>= hGetContents
 
--- | Write a 'ByteString' to a file.
-writeFile :: FilePath -> ByteString -> IO ()
-writeFile f txt = bracket (openFile f WriteMode) hClose
-    (\hdl -> hPut hdl txt)
-
--- | Append a 'ByteString' to a file.
-appendFile :: FilePath -> ByteString -> IO ()
-appendFile f txt = bracket (openFile f AppendMode) hClose
-    (\hdl -> hPut hdl txt)
+-- | Write a ByteString to a handle, appending a newline byte
+--
+hPutStrLn :: Handle -> ByteString -> IO ()
+hPutStrLn h ps = hPut h ps >> hPut h (L.singleton 0x0a)
 
+-- | Write a ByteString to stdout, appending a newline byte
+--
+putStrLn :: ByteString -> IO ()
+putStrLn = hPutStrLn stdout
 
 -- ---------------------------------------------------------------------
 -- Internal utilities
